#pragma once

#ifndef __TOAST_LOGGER_H__
#define __TOAST_LOGGER_H__

#include <stdlib.h>
#include <memory>

#include "sdkVersion.h"

#include "ToastLoggerConfiguration.h"
#include "ToastLoggerUserFields.h"

#ifdef TOASTLOGGER_STATIC_LIBRARY
    #pragma comment(lib, "wininet.lib")
    #define TOASTLOGGER_API
#else
    #ifdef TOASTLOGGER_EXPORTS
        #define TOASTLOGGER_API __declspec(dllexport)
        #define DLLEXPORT_TEMPLATE
    #else
        #define TOASTLOGGER_API __declspec(dllimport)
        #define DLLEXPORT_TEMPLATE extern
    #endif
#endif

#define _DEPRECATED_(since, replacement) __declspec(deprecated("Since " #since "; use " #replacement))

typedef enum _TOAST_LOGGER_LEVEL
{
    LOG_LEVEL_DEBUG = 0,
    LOG_LEVEL_INFO = 1,
    LOG_LEVEL_WARN = 2,
    LOG_LEVEL_ERROR = 3,
    LOG_LEVEL_FATAL = 4,
    LOG_LEVEL_NONE = 5
} TOAST_LOGGER_LEVEL;


//============================================================================================================================
// TOAST SDK Internal APIs
//============================================================================================================================

typedef void(*LogNCrashCallbackType)(void* data);

/// <summary>
/// Setting the process's Unhandled Exception Handler (used internally by SDK)
/// </summary>
/// <param name="exinfo">Parameter passed to SetUnhandledExceptionFilter </param>
/// <returns>Address of previously used exception handler, if NULL, it means that there is no current top-level exception handler.</returns>
extern "C" TOASTLOGGER_API LONG WINAPI  GetHandleException(EXCEPTION_POINTERS * exinfo);

/// <summary>
/// Set error handlers for pure virtual function calls (used internally by SDK)
/// </summary>
/// <returns>Previous _purecall_handler. Returns nullptr if there is no previous handler.</returns>
extern "C" TOASTLOGGER_API _purecall_handler GetHandlePureVirtualCall();

/// <summary>
/// Set a function to be called when the CRT finds an invalid argument (used internally by SDK)
/// </summary>
/// <returns> Previous _invalid_parameter_handler. Returns nullptr if there is no previous handler.</returns>
extern "C" TOASTLOGGER_API _invalid_parameter_handler  GetHandleInvalidParameterCall();

/// <summary>
/// Handling exceptions occurring in threads (used internally by SDK)
/// </summary>
/// <returns> Previous terminate_handler. Returns nullptr if there is no previous handler.</returns>
extern "C" TOASTLOGGER_API terminate_handler  GetHandleTerminateCall();

/// <summary>
/// Handling exceptions occurring in threads (used internally by SDK)
/// </summary>
/// <returns> Previous unexpected_handler. Returns nullptr if there is no previous handler.</returns>
extern "C" TOASTLOGGER_API unexpected_handler  GetHandlUnexpectedCall();

//============================================================================================================================
// TOAST SDK ToastLogger class API
//============================================================================================================================

namespace toast
{
    namespace logger
    {

        /// <summary>
        /// This is an API for using NHN Cloud Log & Crash Search.
        /// Refer to the ToastLoggerConfiguration class for configuration information required for initialization.
        /// </summary>
        class ToastLogger
        {
        public:            
            TOASTLOGGER_API ToastLogger();
            TOASTLOGGER_API  ~ToastLogger();
            TOASTLOGGER_API ToastLogger(ToastLogger&& rhs);
            TOASTLOGGER_API ToastLogger& operator=(ToastLogger&& rhs);

            /// <summary>
            /// Retrieves a singleton instance of the ToastLogger class.
            /// It is the same as GetToastLogger() defined below.
            /// </summary>
            /// <returns>ToastLogger*</returns>
            TOASTLOGGER_API static ToastLogger* GetInstance()
            {
                static ToastLogger loggerInstance;
                return &loggerInstance;
            }

            /// <summary>
            /// Initialize the SDK
            /// </summary>
            /// <param name="loggerConfiguration">Setting information used for initialization./param>
            /// <returns>If initialization is successful, ture is returned, and if it fails, false is returned.</returns>
            bool __forceinline initialize(ToastLoggerConfiguration* loggerConfiguration)
            {
                bool bRet = initializeImplementation(loggerConfiguration);

                if (bRet && loggerConfiguration->isEnableCrashReporter())
                    SetCrashHandler();

                return bRet;
            }

            /// <summary>
            /// Set the User ID.
            /// </summary>
            /// <param name="userId">User ID of char* type.</param>
            TOASTLOGGER_API void setUserId(const char* userId);

            /// <summary>
            /// Set the User ID.
            /// </summary>
            /// <param name="userId">User ID of wchar_t* type.</param>
            TOASTLOGGER_API void setUserId(const wchar_t* userId);

            /// <summary>
            /// Get the User ID.
            /// </summary>
            /// <returns>User ID of char* type.</returns>
            TOASTLOGGER_API const char* getUserId();

            /// <summary>
            /// Get the User ID.
            /// </summary>
            /// <returns>User ID of wchar_t* type.</returns>
            TOASTLOGGER_API const wchar_t* getUserIdW();

            /// <summary>
            /// Set the callback function to be called when a crash occurs.
            /// </summary>
            /// <param name="cb">Callback function to be called when a crash occurs.</param>
            /// <param name="cbData">Data pointer to be referenced in callback function.</param>
            TOASTLOGGER_API void setCrashCallback(const LogNCrashCallbackType cb, void* cbData = NULL);

            /// <summary>
            /// Defines a custom key/value to be added to the log when a crash occurs.
            /// </summary>
            /// <param name="key"> Custom fields to add to the crash log. </param>
            /// <param name="value">Custom value of char* type.</param>
            TOASTLOGGER_API bool addCrashField(const char* key, const char* value);

            /// <summary>
            /// Defines a custom key/value to be added to the log when a crash occurs.
            /// </summary>
            /// <param name="key"> Custom fields to add to the crash log. </param>
             /// <param name="value">Custom value of wchar_t* type.</param>
            TOASTLOGGER_API bool addCrashField(const char* key, const wchar_t* value);

            /// <summary>
            /// Defines a custom key/value to be added to the log message.
            /// Once set, the custom field will continue to be used when sending logs.
            /// Custom field names (key values) start with "A-Z, a-z" and can contain the characters "A-Z, a-z, 0-9, -, _".
            /// https://docs.toast.com/ko/Analytics/Log%20&%20Crash%20Search/ko/api-guide/
            /// </summary>
            /// <param name="key"> Key of the custom field to add to the log </param>
            /// <param name="value">Custom value of char* type. </param>
            /// <returns>Returns false if the key value does not match the format.</returns>
            TOASTLOGGER_API bool addUserField(const char* key, const char* value);

            /// <summary>
            /// Defines a custom key/value to be added to the log message.
            /// Once set, the custom field will continue to be used when sending logs.
            /// Custom field names (key values) start with "A-Z, a-z" and can contain the characters "A-Z, a-z, 0-9, -, _".
            ///  https://docs.toast.com/ko/Analytics/Log%20&%20Crash%20Search/ko/api-guide/
            /// </summary>
            /// <param name="key"> Key of the custom field to add to the log </param>
            /// <param name="value">Custom value of wchar_t* type. </param>
            /// <returns>Returns false if the key value does not match the format.</returns>
            TOASTLOGGER_API bool addUserField(const char* key, const wchar_t* value);

            /// <summary>
            /// Delete custom keys to be added to the log.
            /// </summary>
            /// <param name="key">Key for custom field.<param>
            TOASTLOGGER_API void removeUserField(const char* key);

            /// <summary>
            /// Delete all custom keys.
            /// </summary>
            TOASTLOGGER_API void clearUserFileds();

            /// <summary>
            /// Send the debug/info/warn/error/fatal log.
            /// </summary>
            /// <param name="message">Log message to be sent to NHN Log & Crash.</param>
            /// <param name="userFields">Pass a pointer to the helper class of the custom field. Refer to CreateToastLoggerUserFields() API.
            /// It can be used in the same way as using the addUserField() function.
            /// If the key defined in addUserField() and the key defined in ToastLoggerUserFields are the same, ToastLoggerUserFields takes precedence.
            /// </param>
            TOASTLOGGER_API void debug(const char* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void debug(const wchar_t* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void info(const char* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void info(const wchar_t* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void warn(const char* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void warn(const wchar_t* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void error(const char* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void error(const wchar_t* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void fatal(const char* message, ToastLoggerUserFields* userFields = NULL);
            TOASTLOGGER_API void fatal(const wchar_t* message, ToastLoggerUserFields* userFields = NULL);

            /// <summary>
            /// Send the log.
            /// </summary>
            /// <param name="logLevel">TOAST_LOGGER_LEVEL</param>
            /// <param name="message">Log message to be sent to NHN Log & Crash.</param>
            /// <param name="userFields">Pass a pointer to the helper class of the custom field. Refer to CreateToastLoggerUserFields() API.
            /// It can be used in the same way as using the addUserField() function.
            /// If the key defined in addUserField() and the key defined in ToastLoggerUserFields are the same, ToastLoggerUserFields takes precedence.
            /// </param>
            TOASTLOGGER_API void log(TOAST_LOGGER_LEVEL logLevel, const char* message, ToastLoggerUserFields* userFields = nullptr);
            TOASTLOGGER_API void log(TOAST_LOGGER_LEVEL logLevel, const wchar_t* message, ToastLoggerUserFields* userFields = nullptr);

            /// <summary>
            /// Set up an exception handler. During initialization, it is automatically processed according to the options set by the developer.
            /// </summary>
            __forceinline void SetCrashHandler()
            {
                _previousFilter = SetUnhandledExceptionFilter(GetHandleException);
                _previousPurecallHandler = _set_purecall_handler(GetHandlePureVirtualCall());
                _previousInvalidParameterHandler = _set_invalid_parameter_handler(GetHandleInvalidParameterCall());

                //_previousTerminateHandler = set_terminate(GetHandleTerminateCall());
                //_previousUnexpectedHandler = set_unexpected(GetHandlUnexpectedCall());
            }

            /// <summary>
            /// Release the exception handler. It is handled automatically when the process containing the SDK is terminated.
            /// </summary>
            __forceinline void ResetCrashHandler()
            {
                SetUnhandledExceptionFilter(_previousFilter);
                _set_purecall_handler(_previousPurecallHandler);
                _set_invalid_parameter_handler(_previousInvalidParameterHandler);

                //set_terminate(_previousTerminateHandler);
                //set_unexpected(_previousUnexpectedHandler);
            }

        private:
            TOASTLOGGER_API bool initializeImplementation(ToastLoggerConfiguration* loggerConfiguration) ;
           
            class LoggerImpl;
            std::unique_ptr<LoggerImpl>    pImpl;

            LPTOP_LEVEL_EXCEPTION_FILTER    _previousFilter;
            _purecall_handler               _previousPurecallHandler;
            _invalid_parameter_handler      _previousInvalidParameterHandler;
            //terminate_handler               _previousTerminateHandler;
            //unexpected_handler              _previousUnexpectedHandler;
        };

    }
}

//============================================================================================================================
// TOAST SDK Global API
//============================================================================================================================

/// <summary>
/// [Deprecated API]Retrieves a singleton instance of the ToastLogger class. Can be replaced with ToastLogger::GetInstance() function.
/// </summary>
/// <returns>A pointer to the ToastLogger class.</returns>
extern "C" TOASTLOGGER_API toast::logger::ToastLogger * GetToastLogger();

/// <summary>
/// [Deprecated API]Function to release the crash handler. Only the crash handler is released.
/// </summary>
/// <returns>void</returns>
extern "C" TOASTLOGGER_API void DestroyToastLogger();


#endif  // __TOAST_LOGGER_H__