#pragma once

#ifndef __TOAST_LOGGER_CONFIGURATION__
#define __TOAST_LOGGER_CONFIGURATION__

#include <stdlib.h>
#include <memory>

#ifdef TOASTLOGGER_STATIC_LIBRARY
#pragma comment(lib, "wininet.lib")
#define TOASTLOGGER_API
#else
#ifdef TOASTLOGGER_EXPORTS
#define TOASTLOGGER_API __declspec(dllexport)
#define DLLEXPORT_TEMPLATE
#else
#define TOASTLOGGER_API __declspec(dllimport)
#define DLLEXPORT_TEMPLATE extern
#endif
#endif

#define _DEPRECATED_(since, replacement) __declspec(deprecated("Since " #since "; use " #replacement))



typedef enum _TOAST_SERVICE_ZONE
{
    TOAST_SERVICE_ZONE_ALPHA = 0,
    TOAST_SERVICE_ZONE_BETA = 1,
    TOAST_SERVICE_ZONE_REAL = 2
} TOAST_SERVICE_ZONE;

typedef enum _TOAST_LANGUAGE_TYPE
{
    TOAST_LANGUAGE_ENGLISH = 1,
    TOAST_LANGUAGE_KOREAN = 2
} TOAST_LANGUAGE_TYPE;

namespace toast
{
    namespace logger
    {
        /// <summary>
        /// This is the setting information used when initializing the NHN Cloud SDK.
        /// Set the Project Version, AppKey, etc., and decide whether to use the crash report function.
        /// </summary>
        class ToastLoggerConfiguration
        {
        public:
            TOASTLOGGER_API ToastLoggerConfiguration();
            TOASTLOGGER_API ~ToastLoggerConfiguration();
            TOASTLOGGER_API ToastLoggerConfiguration(ToastLoggerConfiguration&& rhs);
            TOASTLOGGER_API ToastLoggerConfiguration& operator=(ToastLoggerConfiguration&& rhs);

            /// <summary>
            /// Retrieves a singleton instance of the ToastLoggerConfiguration class.
            /// It is the same as GetToastLoggerConfiguration() defined below.
            /// </summary>
            /// <returns>ToastLoggerConfiguration*</returns>
            TOASTLOGGER_API static ToastLoggerConfiguration* GetInstance()
            {
                static ToastLoggerConfiguration configurationInstance;
                return &configurationInstance;
            }
            /// <summary>
            /// Get project version information.
            /// </summary>
            /// <returns>char*</returns>
            TOASTLOGGER_API const char* getProjectVersion();

            /// <summary>
            /// Set project version information. 
            /// It must match the version entered when registering the symbol file in the Log&Crash Search console.
            /// </summary>
            /// <param name="projectVersion"></param>
            TOASTLOGGER_API void setProjectVersion(const char* projectVersion);

            /// <summary>
            /// Gets NHN Cloud Log & Crash Search's AppKey.
            /// </summary>
            /// <returns>char*</returns>
            TOASTLOGGER_API const char* getProjectKey();

            /// <summary>
            /// Set the NHN Cloud Log & Crash Search's AppKey.
            /// App key can be obtained from Log&Crash Search console.
            /// </summary>
            /// <param name="projectKey"></param>
            TOASTLOGGER_API void setProjectKey(const char* projectKey);

            /// <summary>
            /// Returns whether the Crash Reporting function is used.
            /// </summary>
            /// <returns>boolean : Whether to use the crash reporter set by enableCrashReporter()</returns>
            TOASTLOGGER_API bool isEnableCrashReporter(void);
            TOASTLOGGER_API _DEPRECATED_(1.0, isEnableCrashReporter) bool enableCrashReporter();

            /// <summary>
            /// Set whether to use the crash reporting function. When an exception occurs, a crash dump is generated and reported.
            /// It support SEH(SetUnhandledExceptionFilter), _set_purecall_handler, _set_invalid_parameter_handler.
            /// <summary>
            TOASTLOGGER_API void enableCrashReporter(bool enableCrashReporter);

            /// <summary>
            /// Returns whether crash dumps are sent by the executable file containing the SDK.
            /// </summary>
            /// <returns>boolean, true μ  ũ Ʈ , false ܺ Ϸ ũ  </returns>
            TOASTLOGGER_API bool isEnableCrashSilenceMode();
            TOASTLOGGER_API _DEPRECATED_(1.0, isEnableCrashSilenceMode)  bool enableCrashSilenceMode();

            /// <summary>
            /// Set whether to send a crash report to an external process (executable file) when a crash occurs.
            /// If set to false, crash dump is sent to an external executable file (CrashReporter.exe).
            /// </summary>
            TOASTLOGGER_API void enableCrashSilenceMode(bool enableCrashSilenceMode);

            /// <summary>
            /// Sets whether or not the external executable file (CrashReporter.exe) exposes the UI.
            /// </summary>
            /// <returns>boolean, If true, expose the CrashReporter.exe's UI</returns>
            TOASTLOGGER_API bool isExposeExternalCrashReporterUI();

            /// <summary>
            /// Affected when enableCrashSilenceMode(false).
            /// Set whether to expose the UI of CrashReporter.exe.
            /// Additional messages can be entered through the input window of the user dialog, and the report is sent when the user clicks the 'ok' button.
            /// If the user selects 'cancel', the crash report is not sent.
            /// </summary>
            TOASTLOGGER_API void exposeExternalCrashReporterUI(bool expose);

            /// <summary>
            /// Affected when isExposeExternalCrashReporterUI(true).
            /// Set the message to be displayed on the reporting UI of the external executable file (CrashReporter.exe)
            /// </summary>
            /// <param name="langType"> Language code of the language to be displayed</param>
            /// <param name="message"> Message to be displayed </param>
            TOASTLOGGER_API void setExternalCrashReporterMessage(TOAST_LANGUAGE_TYPE langType, const wchar_t* message);

            /// <summary>
            /// Returns whether personal information (IP Address) is to be collected.
            /// </summary>
            /// <returns>If true, no personal information is collected. </returns>
            TOASTLOGGER_API bool isEnableApplyGDPR();
            TOASTLOGGER_API _DEPRECATED_(1.0, isEnableApplyGDPR) bool enableApplyGDPR();

            /// <summary>
            /// Set whether to collect personal information(IP Address).
            /// Determines whether to collect the endpoint IPs that send logs. The default is set to false.
            /// If set to enableApplyGDPR(true), the IP of the endpoint sending the log is fixed as "127.0.0.1"
            /// </summary>
            /// <param name="appyGDPR">If true, no personal information is collected. </param>
            TOASTLOGGER_API void enableApplyGDPR(bool appyGDPR);

            /// <summary>
            /// [Generally not used] Get NHN Cloud service zone.
            /// </summary>
            /// <returns>TOAST_SERVICE_ZONE code</returns>
            TOASTLOGGER_API TOAST_SERVICE_ZONE getServiceZone();

            /// <summary>
            /// [Generally not used] Set the NHN Cloud service zone to be used.
            /// </summary>
            TOASTLOGGER_API void setServiceZone(TOAST_SERVICE_ZONE serviceZone);

        private:
            class ConfigurationImpl;
            std::unique_ptr<ConfigurationImpl>     pConfImpl;
        };
    }
}


//============================================================================================================================
// TOAST SDK Global API
//============================================================================================================================

extern "C" TOASTLOGGER_API toast::logger::ToastLoggerConfiguration * GetToastLoggerConfiguration();


#endif  //__TOAST_LOGGER_CONFIGURATION__