#pragma once

#ifndef __TOAST_LOGGER_USERFIELDS__
#define __TOAST_LOGGER_USERFIELDS__

#ifdef TOASTLOGGER_STATIC_LIBRARY
    #define TOASTLOGGER_API
#else
    #ifdef TOASTLOGGER_EXPORTS
    #define TOASTLOGGER_API     __declspec(dllexport)
    #else
    #define TOASTLOGGER_API     __declspec(dllimport)
    #endif
#endif  // TOASTLOGGER_STATIC_LIBRARY


#include <windows.h>
#include <stdio.h>
#include <malloc.h>
#include <memory>
#include <map>
#include <string>

typedef std::map<std::string, std::wstring> MapString;


namespace toast
{
    namespace logger
    {
        /// <summary>
        /// This is a helper class that provides more convenient use of user-defined fields.
        /// It provides a more flexible usage environment than user-defined keys/values through addUserField(), removeUserField(), clearUserFileds().
        /// </summary>
        class ToastLoggerUserFields
        {
        public:
            TOASTLOGGER_API ToastLoggerUserFields();
            TOASTLOGGER_API ~ToastLoggerUserFields();
            TOASTLOGGER_API ToastLoggerUserFields(ToastLoggerUserFields&& rhs);
            TOASTLOGGER_API ToastLoggerUserFields& operator=(ToastLoggerUserFields&& rhs);

            /// <summary>
            /// Get ToastLoggerUserFields's instance(Singleton).  
            /// It is the same as GetToastLoggerConfiguration() defined below.
            /// </summary>
            /// <returns>ToastLoggerConfiguration*</returns>
            TOASTLOGGER_API static ToastLoggerUserFields* GetInstance()
            {
                static ToastLoggerUserFields userFieldInstance;
                return &userFieldInstance;
            }

            /// <summary>
            /// Set the key/value value of the custom field to be sent with the log.
            /// Custom field names (key values) start with "A-Z, a-z" and can contain the characters "A-Z, a-z, 0-9, -, _".
            ///  https://docs.toast.com/ko/Analytics/Log%20&%20Crash%20Search/ko/api-guide/
            /// </summary>
            /// <param name="key">Key value for custom field</param>
            /// <param name="value">Custom field values</param>
            TOASTLOGGER_API bool insert(const char* key, const char* value);
            TOASTLOGGER_API bool insert(const char* key, const wchar_t* value);

            /// <summary>
            /// Deletes the key and value corresponding to a specific key.
            /// </summary>
            /// <param name="key">y to be deleted</param>
            TOASTLOGGER_API void erase(const char* key);

            /// <summary>
            /// Check if the key exists.
            /// </summary>
            /// <param name="key"></param>
            /// <returns>bool : Returns true if key exists</returns>
            TOASTLOGGER_API bool find(const char* key);
            
            /// <summary>
            /// Return the number of custom Key/Value pairs managed by the ToastLoggerUserFields class.
            /// </summary>
            /// <returns>int : Number of key/value pairs managed by ToastLoggerUserFields class.</returns>
            TOASTLOGGER_API int size();

            /// <summary>
            /// Check if the custom Key/Value is empty.
            /// </summary>
            /// <returns>bool : Returns number of Key/Value pair.</returns>
            TOASTLOGGER_API bool empty();

            /// <summary>
            /// Empty the custom Key/Value pairs.
            /// </summary>
            TOASTLOGGER_API void clear();

            /// <summary>
            /// Returns Map structure in Key/value format defined as MapString.
            /// </summary>
            /// <returns>MapString</returns>
            TOASTLOGGER_API MapString& getMap();
        private:
            class UserFieldsImpl;
            std::unique_ptr<UserFieldsImpl>     pUserFieldsImpl;

        };
    }
}

/// <summary>
/// It returns the ToastLoggerUserFields class for flexible use of custom fields.
/// </summary>
/// <returns>ToastLoggerUserFields*</returns>
extern "C" TOASTLOGGER_API toast::logger::ToastLoggerUserFields * CreateToastLoggerUserFields();

#endif  //__TOAST_LOGGER_USERFIELDS__